@@ -50,8 +50,8 @@ def number(v):
         return int(v)
 
 _STOP_CHARS = [" ", ",", "\n", "\r", "\t"]
-_COLL_OPEN_CHARS = ["#", "[", "{"]
-_COLL_CLOSE_CHARS = ["]", "}"]
+_COLL_OPEN_CHARS = ["#", "[", "{", "("]
+_COLL_CLOSE_CHARS = ["]", "}", ")"]
 _EXTRA_NUM_CHARS = ["-", "+", ".", "e", "E"]
 
 class CljDecoder(object):
@@ -220,6 +220,7 @@ class CljEncoder(object):
     def __init__(self, data, fd):
         self.data = data
         self.fd = fd
+        self.circular = {}
 
     def encode(self):
         self.__do_encode(self.data)
@@ -231,7 +232,7 @@ class CljEncoder(object):
             return ("string", False)
         elif isinstance(t, bool):
             return ("boolean", False)
-        elif isinstance(t, float) or isinstance(t, int):
+        elif isinstance(t, (int,float,long)):
             return ("number", False)
         elif isinstance(t, dict):
             return ("dict", True)
@@ -247,6 +248,12 @@ class CljEncoder(object):
         t,coll = self.get_type(d)
 
         if coll:
+            refid = id(d)
+            if refid in self.circular:
+                raise ValueError('Circular reference detected')
+            else:
+                self.circular[refid] = True
+            
             if t == "dict":
                 fd.write("{")
                 for k,v in d.items():
